((window, $, Drupal, debounce, countUp) => {
  /**
   * Attach the countUp.js behavior to the expected elements.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.countupformatter = {
    /**
     * The selector to be used for CountUp targets.
     *
     * @var string
     */
    stepSelector: '.countup-formatter',

    /**
     * {@inheritDoc}
     */
    attach: function attach() {
      if ($(once('countup', 'body').length)) {
        $(window).on(
          'DOMContentLoaded load resize scroll',
          debounce(this.checkVisibility.bind(this), 50),
        );
      }
    },

    checkVisibility: function checkVisibility() {
      const o = this;
      const $elements = once(
        'countupElements',
        `${this.stepSelector}:not(.countup-processed)`,
      );

      // Define an observer instance
      const observer = new IntersectionObserver(
        function (entries) {
          // Callback is called on intersection change.
          entries.forEach(function (entry) {
            if (entry.isIntersecting) {
              const element = entry.target;
              $(element).addClass('countup-processed');
              o.countup(element);
              observer.unobserve(element);
            }
          });
        },
        {
          // Default is the viewport.
          root: null,
          // Percentage of target's visible area. Triggers "onIntersection".
          threshold: 0.5,
        },
      );
      $elements.forEach(function (element) {
        // Use the observer to observe an element
        observer.observe(element);
      });
    },

    /**
     * Performs the countUp action.
     *
     * @param {Element} element
     *   Element to perform action.
     */
    countup: (element) => {
      if (typeof countUp.CountUp !== 'function') {
        return;
      }

      const $element = $(element);
      const options = $.extend(
        { start: 0, duration: 2.5, decimalPlaces: 0 },
        $element.data(),
      );

      if (typeof options.end === 'undefined') {
        if ($.isNumeric($element.html())) {
          options.end = $element.html();
          $element.attr('data-end', $element.html());
        } else {
          // Nothing valid is defined for the "end" value, exit here...
          return;
        }
      }

      // Instantiate the plugin.
      const count = new countUp.CountUp(element, options.end, options);
      if (!count.error) {
        count.start();
      } else if (window.console) {
        window.console.error(count.error);
      }
    },
  };
})(window, jQuery, window.Drupal, window.Drupal.debounce, window.countUp);
